@@ -90,7 +90,7 @@ static const QString channelTypeToIconName( MixDevice::ChannelType type )
         case MixDevice::APPLICATION_TOMAHAWK:
         	return "tomahawk";
         case MixDevice::APPLICATION_CLEMENTINE:
-        	return "clementine";
+        	return "application-x-clementine";
         case MixDevice::APPLICATION_VLC:
         	return "vlc";
 
@@ -127,9 +127,8 @@ void MixDevice::init(  Mixer* mixer, const QString& id, const QString& name, con
     _mixer = mixer;
     _id = id;
     _enumCurrentId = 0;
-    mediaPlayControl = false;
-    mediaNextControl = false;
-    mediaPrevControl = false;
+
+    mediaController = new MediaController(_id);
     if( name.isEmpty() )
         _name = i18n("unknown");
     else
@@ -162,6 +161,12 @@ void MixDevice::close()
 }
 
 
+MediaController* MixDevice::getMediaController()
+{
+	return mediaController;
+}
+
+
 shared_ptr<MixDevice> MixDevice::addToPool()
 {
 //	kDebug() << "id=" <<  _mixer->id() << ":" << _id;
@@ -209,7 +214,8 @@ void MixDevice::increaseOrDecreaseVolume(bool decrease, Volume::VolumeTypeFlag v
 
 	if (volumeType & Volume::Capture)
 	{
-		kDebug() << "VolumeType=" << volumeType << "   c";
+		if (debugme)
+			kDebug() << "VolumeType=" << volumeType << "   c";
 
 		Volume& volC = captureVolume();
 		long inc = volC.volumeStep(decrease);
