@@ -38,6 +38,10 @@
 #include <QtCore/qstring.h>
 
 QMap<QString, GUIProfile*> GUIProfile::s_profiles;
+QString const GUIProfile::PNameSimple("simple");
+QString const GUIProfile::PNameExtended("extended");
+QString const GUIProfile::PNameAll("all");
+QString const GUIProfile::PNameCustom("custom");
 
 bool SortedStringComparator::operator()(const std::string& s1, const std::string& s2) const {
     return ( s1 < s2 );
@@ -149,12 +153,14 @@ QString GUIProfile::buildReadableProfileName(Mixer* mixer, QString profileName)
     QString fname;
     fname += mixer->getBaseName();
     if ( mixer->getCardInstance() > 1 ) {
-        fname += ' ' + mixer->getCardInstance();
+        fname += " %1";
+        fname = fname.arg(mixer->getCardInstance());
     }
     if ( profileName != "default" ) {
         fname += ' ' + profileName;
     }
 
+    kDebug() << fname;
     return fname;
 }
 
@@ -269,10 +275,8 @@ void GUIProfile::addProfile(GUIProfile* guiprof)
 GUIProfile* GUIProfile::loadProfileFromXMLfiles(Mixer* mixer, QString profileName)
 {
     GUIProfile* guiprof = 0;
-    QString fileName, fileNameFQ;
-
-    fileName = "profiles/" + profileName + ".xml";
-    fileNameFQ = KStandardDirs::locate("appdata", fileName );
+    QString fileName = createNormalizedFilename(profileName);
+    QString fileNameFQ = KStandardDirs::locate("appdata", fileName );
 
     if ( ! fileNameFQ.isEmpty() ) {
         guiprof = new GUIProfile();
@@ -367,14 +371,22 @@ bool GUIProfile::readProfile(const QString& ref_fileName)
     return ok;
 }
 
+const QString GUIProfile::createNormalizedFilename(const QString& profileId)
+{
+	QString profileIdNormalized(profileId);
+	profileIdNormalized.replace(':', '.');
+
+	QString fileName("profiles/");
+	fileName = fileName + profileIdNormalized + ".xml";
+	return fileName;
+ }
 
 bool GUIProfile::writeProfile()
 {
    bool ret = false;
-   QString fileName, fileNameFQ;
-   fileName = "profiles/" + getId() + ".xml";
-   fileName.replace(':', '.');
-   fileNameFQ = KStandardDirs::locateLocal("appdata", fileName, true );
+   QString profileId = getId();
+   QString fileName = createNormalizedFilename(profileId);
+   QString fileNameFQ = KStandardDirs::locateLocal("appdata", fileName, true );
 
    kDebug() << "Write profile:" << fileNameFQ ;
    QFile f(fileNameFQ);
@@ -399,12 +411,31 @@ bool GUIProfile::finalizeProfile() const
     return ok;
 }
 
+
+// -------------------------------------------------------------------------------------
 void GUIProfile::setControls(ControlSet& newControlSet)
 {
     qDeleteAll(_controls);
     _controls = newControlSet;
 }
 
+const GUIProfile::ControlSet& GUIProfile::getControls() const
+{
+    return _controls;
+}
+
+GUIProfile::ControlSet& GUIProfile::getControls()
+{
+    return _controls;
+}
+
+void GUIProfile::addProduct(ProfProduct* prd)
+{
+	_products.insert(prd);
+}
+
+// -------------------------------------------------------------------------------------
+
 
 /**
  * Returns how good the given Mixer matches this GUIProfile.
@@ -615,6 +646,11 @@ ProfControl::~ProfControl() {
     delete d;
 }
 
+void ProfControl::setVisible(bool visible)
+{
+	show = visible ? GUIProfile::PNameSimple : GUIProfile::PNameExtended;
+}
+
 void ProfControl::setSubcontrols(QString sctls)
 {
     d->subcontrols = sctls;
@@ -807,7 +843,7 @@ void GUIProfileParser::addProduct(const QXmlAttributes& attributes) {
 		prd->productRelease = release;
 		prd->comment = comment;
 
-		_guiProfile->_products.insert(prd);
+		_guiProfile->addProduct(prd);
 	}
 }
 
